==> llm-docs-0.1.2.tgz: package/docs/testing/strategies.md <==
---
id: testing-strategies
---
# Testing Strategies Guide

<!-- title: Testing Strategies Guide -->
<!-- description: Practical guide for testing strategies in the Bluefly Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers practical testing strategies for the Bluefly Platform, including unit testing, integration testing, end-to-end testing, and performance testing.

## 🎯 Drupal TDD Standardization

### Objective

Standardize Test-Driven Development (TDD) across all Drupal modules using ReelCode integration for consistent code quality and testing patterns.

### Current State Analysis

#### Modules Requiring TDD Standardization

1. **✅ ai_swarm_intelligence** - COMPLETED (100% TDD compliant)
2. **🔄 drupal_migration_plus** - IN PROGRESS (dual test structure)
3. **📋 field_eca_bridge** - NEEDS TDD structure
4. **📋 llm** - NEEDS TDD structure  
5. **📋 langchain** - NEEDS TDD structure
6. **📋 api_normalization** - NEEDS TDD structure
7. **📋 secure_drupal** - NEEDS TDD structure
8. **📋 alternative_services** - NEEDS TDD structure
9. **📋 blockchain_manager** - NEEDS TDD structure
10. **📋 experience_builder_converter** - NEEDS TDD structure
11. **📋 mcp_client_extras** - NEEDS TDD structure
12. **📋 marketplace_integration** - NEEDS TDD structure
13. **📋 native_ai** - NEEDS TDD structure
14. **📋 recipe_onboarding** - NEEDS TDD structure

### ReelCode Integration Strategy

#### Phase 1: ReelCode Enhancement for Drupal TDD

```typescript
// Extend ReelCode with Drupal-specific validators
export interface DrupalTDDConfig extends StandardsConfig {
  drupal: {
    moduleStructure: boolean;
    testCoverage: number; // 95% minimum
    tddWorkflow: boolean;
    serviceInjection: boolean;
    ecaIntegration: boolean;
  };
  php: {
    psr12: boolean;
    drupalStandards: boolean;
    phpstan: boolean;
  };
}
```

#### Phase 2: Automated TDD Structure Generation

```bash
# Generate standardized TDD structure for any module
reelcode drupal:generate-tdd-structure --module=module_name

# Validate TDD compliance
reelcode drupal:validate-tdd --module=module_name

# Generate missing tests
reelcode drupal:generate-tests --module=module_name --coverage=95
```

### Standardized TDD Structure

#### Every Module Must Have

```
module_name/
├── module_name.info.yml
├── module_name.module
├── module_name.services.yml
├── phpunit.xml.dist
├── composer.json
├── README.md
├── src/
│   ├── Service/
│   │   ├── Controller/
│   │   ├── Form/
│   │   ├── Plugin/
│   │   │   └── ECA/
│   │   │       ├── Action/
│   │   │       └── Condition/
│   │   └── Entity/
│   ├── tests/
│   │   ├── src/
│   │   │   ├── Unit/
│   │   │   │   ├── Service/
│   │   │   │   ├── Plugin/
│   │   │   │   └── Entity/
│   │   │   ├── Kernel/
│   │   │   ├── Functional/
│   │   │   └── Integration/
│   │   ├── utils/
│   │   └── bootstrap.php
│   ├── config/
│   ├── css/
│   ├── js/
│   └── templates/
```

#### Test File Naming Conventions

- **Unit Tests**: `{ClassName}Test.php`
- **Integration Tests**: `{Feature}IntegrationTest.php`
- **Functional Tests**: `{Feature}FunctionalTest.php`
- **TDD Utilities**: `{Purpose}TddRunner.php`

### Implementation Plan

#### Step 1: Complete drupal_migration_plus TDD Standardization

- [ ] Consolidate dual test structure (`__tests__/` + `tests/`)
- [ ] Organize tests by type (Unit, Kernel, Functional, Integration)
- [ ] Standardize PHPUnit configuration
- [ ] Create TDD utilities

#### Step 2: Enhance ReelCode for Drupal TDD

- [ ] Add Drupal-specific validation rules
- [ ] Create TDD structure generator
- [ ] Add test coverage validation
- [ ] Integrate with PHPStan and PHPCS

#### Step 3: Automated Module Standardization

- [ ] Create script to standardize all remaining modules
- [ ] Generate missing test structures
- [ ] Validate TDD compliance
- [ ] Create CI/CD integration

#### Step 4: Cross-Module Integration Testing

- [ ] Create integration tests between modules
- [ ] Test ECA workflow integration
- [ ] Validate AI platform integration
- [ ] Performance testing

### Quality Gates

#### TDD Compliance Requirements

- ✅ **95%+ Test Coverage** for all modules
- ✅ **Test-First Development** for all new features
- ✅ **Proper Test Organization** by type
- ✅ **Service Dependency Injection** validation
- ✅ **ECA Integration** testing
- ✅ **PHPStan Level 8** compliance
- ✅ **PSR-12 + Drupal Standards** compliance

#### ReelCode Integration

- ✅ **Automated TDD Structure** generation
- ✅ **Test Coverage** validation
- ✅ **Code Quality** enforcement
- ✅ **CI/CD Integration** validation

### Next Actions

1. **Complete drupal_migration_plus standardization** (current)
2. **Enhance ReelCode with Drupal TDD capabilities**
3. **Create automated standardization script**
4. **Apply to all remaining modules**
5. **Implement cross-module integration testing**

### Success Metrics

- **100% TDD Compliance** across all modules
- **95%+ Test Coverage** maintained
- **Consistent Development Patterns** across team
- **Automated Quality Enforcement** via ReelCode
- **Faster Development Velocity** with TDD workflow

### ReelCode CLI Commands for TDD

```bash
# Validate TDD compliance
reelcode drupal validate-tdd -m /path/to/module -c 95

# Generate TDD structure
reelcode drupal generate-tdd-structure -m /path/to/module

# Standardize all modules in directory
reelcode drupal standardize-all -d /path/to/modules

# Generate missing tests
reelcode drupal generate-tests -m /path/to/module -t unit
```

### Integration with CI/CD

```bash
# In your CI pipeline
npm install -g @bluefly/reelcode
reelcode drupal validate-tdd -m ./modules/custom/my_module -c 95
```

---

## Unit Testing

### Agent Testing

```typescript
// src/services/agent.service.spec.ts
describe('AgentService', () => {
  let service: AgentService;
  let modelService: ModelService;
  let memoryService: MemoryService;
  
  beforeEach(async () => {
    const module = await Test.createTestingModule({
      providers: [
        AgentService,
        {
          provide: ModelService,
          useValue: {
            execute: jest.fn(),
            train: jest.fn()
          }
        },
        {
          provide: MemoryService,
          useValue: {
            store: jest.fn(),
            search: jest.fn()
          }
        }
      ]
    }).compile();
    
    service = module.get(AgentService);
    modelService = module.get(ModelService);
    memoryService = module.get(MemoryService);
  });
  
  describe('execute', () => {
    it('should execute agent successfully', async () => {
      const agentId = 'test-agent';
      const input = 'Hello, agent!';
      const config = { model: 'claude-3-opus' };
      
      const expectedResult = {
        output: 'Hello, human!',
        tokens: 10
      };
      
      jest.spyOn(modelService, 'execute')
        .mockResolvedValue(expectedResult);
      
      const result = await service.execute(agentId, input, config);
      
      expect(result).toEqual(expectedResult);
      expect(modelService.execute)
        .toHaveBeenCalledWith(agentId, input, config);
      expect(memoryService.store)
        .toHaveBeenCalledWith(agentId, expectedResult);
    });
    
    it('should handle execution errors', async () => {
      const agentId = 'test-agent';
      const input = 'Hello, agent!';
      const config = { model: 'claude-3-opus' };
      
      const error = new Error('Model error');
      
      jest.spyOn(modelService, 'execute')
        .mockRejectedValue(error);
      
      await expect(service.execute(agentId, input, config))
        .rejects.toThrow('Model error');
      
      expect(memoryService.store)
        .not.toHaveBeenCalled();
    });
  });
});
```

### Model Testing

```typescript
// src/services/model.service.spec.ts
describe('ModelService', () => {
  let service: ModelService;
  let provider: ModelProvider;
  
  beforeEach(async () => {
    const module = await Test.createTestingModule({
      providers: [
        ModelService,
        {
          provide: 'MODEL_PROVIDER',
          useValue: {
            execute: jest.fn(),
            train: jest.fn()
          }
        }
      ]
    }).compile();
    
    service = module.get(ModelService);
    provider = module.get('MODEL_PROVIDER');
  });
  
  describe('execute', () => {
    it('should execute model successfully', async () => {
      const modelId = 'claude-3-opus';
      const input = 'Hello, model!';
      const config = { maxTokens: 100 };
      
      const expectedResult = {
        output: 'Hello, human!',
        tokens: 10
      };
      
      jest.spyOn(provider, 'execute')
        .mockResolvedValue(expectedResult);
      
      const result = await service.execute(modelId, input, config);
      
      expect(result).toEqual(expectedResult);
      expect(provider.execute)
        .toHaveBeenCalledWith(modelId, input, config);
    });
  });
});
```

## Integration Testing

### API Testing

```typescript
// test/integration/agent.test.ts
describe('Agent API', () => {
  let app: INestApplication;
  let agentService: AgentService;
  
  beforeAll(async () => {
    const module = await Test.createTestingModule({
      imports: [AppModule],
      providers: [AgentService]
    }).compile();
    
    app = module.createNestApplication();
    agentService = module.get(AgentService);
    
    await app.init();
  });
  
  afterAll(async () => {
    await app.close();
  });
  
  describe('POST /agents', () => {
    it('should create agent', async () => {
      const input = {
        name: 'Test Agent',
        model: 'claude-3-opus',
        config: {
          maxTokens: 100
        }
      };
      
      const response = await request(app.getHttpServer())
        .post('/agents')
        .send(input)
        .expect(201);
      
      expect(response.body).toMatchObject({
        name: input.name,
        model: input.model,
        status: 'active'
      });
    });
  });
  
  describe('GET /agents/:id', () => {
    it('should get agent', async () => {
      const agent = await agentService.create({
        name: 'Test Agent',
        model: 'claude-3-opus',
        config: {
          maxTokens: 100
        }
      });
      
      const response = await request(app.getHttpServer())
        .get(`/agents/${agent.id}`)
        .expect(200);
      
      expect(response.body).toMatchObject({
        id: agent.id,
        name: agent.name,
        model: agent.model
      });
    });
  });
});
```

### Drupal Testing

```php
// tests/src/Functional/AgentTest.php
namespace Drupal\Tests\bfllm\Functional;

use Drupal\Tests\BrowserTestBase;

class AgentTest extends BrowserTestBase {
  protected $defaultTheme = 'stark';
  
  protected static $modules = ['bfllm'];
  
  public function testAgentCreation() {
    $this->drupalLogin($this->createUser(['administer agents']));
    
    $this->drupalGet('/admin/config/bfllm/agents/add');
    
    $this->submitForm([
      'name' => 'Test Agent',
      'model' => 'claude-3-opus',
      'config[max_tokens]' => 100
    ], 'Save');
    
    $this->assertSession()
      ->pageTextContains('Agent Test Agent has been created.');
  }
  
  public function testAgentExecution() {
    $agent = $this->createAgent([
      'name' => 'Test Agent',
      'model' => 'claude-3-opus',
      'config' => [
        'max_tokens' => 100
      ]
    ]);
    
    $this->drupalGet("/agent/{$agent->id()}/execute");
    
    $this->submitForm([
      'input' => 'Hello, agent!'
    ], 'Execute');
    
    $this->assertSession()
      ->pageTextContains('Execution successful');
  }
}
```

## End-to-End Testing

### Playwright Tests

```typescript
// tests/e2e/agent.spec.ts
import { test, expect } from '@playwright/test';

test.describe('Agent UI', () => {
  test.beforeEach(async ({ page }) => {
    await page.goto('/agents');
  });
  
  test('should create agent', async ({ page }) => {
    await page.click('button:has-text("Create Agent")');
    
    await page.fill('input[name="name"]', 'Test Agent');
    await page.selectOption('select[name="model"]', 'claude-3-opus');
    await page.fill('input[name="maxTokens"]', '100');
    
    await page.click('button:has-text("Save")');
    
    await expect(page.locator('text=Agent created successfully'))
      .toBeVisible();
  });
  
  test('should execute agent', async ({ page }) => {
    await page.click('text=Test Agent');
    
    await page.fill('textarea[name="input"]', 'Hello, agent!');
    await page.click('button:has-text("Execute")');
    
    await expect(page.locator('text=Execution successful'))
      .toBeVisible();
    
    await expect(page.locator('text=Hello, human!'))
      .toBeVisible();
  });
});
```

### Cypress Tests

```typescript
// cypress/integration/agent.spec.ts
describe('Agent UI', () => {
  beforeEach(() => {
    cy.visit('/agents');
  });
  
  it('should create agent', () => {
    cy.get('button').contains('Create Agent').click();
    
    cy.get('input[name="name"]').type('Test Agent');
    cy.get('select[name="model"]').select('claude-3-opus');
    cy.get('input[name="maxTokens"]').type('100');
    
    cy.get('button').contains('Save').click();
    
    cy.contains('Agent created successfully')
      .should('be.visible');
  });
  
  it('should execute agent', () => {
    cy.contains('Test Agent').click();
    
    cy.get('textarea[name="input"]')
      .type('Hello, agent!');
    cy.get('button').contains('Execute').click();
    
    cy.contains('Execution successful')
      .should('be.visible');
    
    cy.contains('Hello, human!')
      .should('be.visible');
  });
});
```

## Performance Testing

### Load Testing

```typescript
// tests/performance/agent.load.ts
import { check } from 'k6';
import http from 'k6/http';

export const options = {
  stages: [
    { duration: '1m', target: 50 },
    { duration: '3m', target: 50 },
    { duration: '1m', target: 0 }
  ],
  thresholds: {
    http_req_duration: ['p(95)<500'],
    http_req_failed: ['rate<0.01']
  }
};

export default function() {
  const response = http.post(
    'http://localhost:3000/agents/execute',
    JSON.stringify({
      agent_id: 'test-agent',
      input: 'Hello, agent!',
      config: {
        model: 'claude-3-opus',
        max_tokens: 100
      }
    }),
    {
      headers: {
        'Content-Type': 'application/json'
      }
    }
  );
  
  check(response, {
    'status is 200': (r) => r.status === 200,
    'response time < 500ms': (r) => r.timings.duration < 500
  });
}
```

### Stress Testing

```typescript
// tests/performance/agent.stress.ts
import { check } from 'k6';
import http from 'k6/http';

export const options = {
  stages: [
    { duration: '2m', target: 100 },
    { duration: '5m', target: 100 },
    { duration: '2m', target: 200 },
    { duration: '5m', target: 200 },
    { duration: '2m', target: 300 },
    { duration: '5m', target: 300 },
    { duration: '2m', target: 0 }
  ],
  thresholds: {
    http_req_duration: ['p(95)<1000'],
    http_req_failed: ['rate<0.05']
  }
};

export default function() {
  const response = http.post(
    'http://localhost:3000/agents/execute',
    JSON.stringify({
      agent_id: 'test-agent',
      input: 'Hello, agent!',
      config: {
        model: 'claude-3-opus',
        max_tokens: 100
      }
    }),
    {
      headers: {
        'Content-Type': 'application/json'
      }
    }
  );
  
  check(response, {
    'status is 200': (r) => r.status === 200,
    'response time < 1s': (r) => r.timings.duration < 1000
  });
}
```

## Best Practices

1. **Unit Testing**
   - Test business logic
   - Mock dependencies
   - Use meaningful assertions
   - Maintain coverage

2. **Integration Testing**
   - Test API endpoints
   - Test database
   - Test external services
   - Use test database

3. **End-to-End Testing**
   - Test user flows
   - Test UI components
   - Test error handling
   - Use test data

4. **Performance Testing**
   - Set realistic goals
   - Monitor resources
   - Analyze results
   - Update thresholds

## Related Documentation

- [Architecture Guide](../architecture/README)
- [Development Guide](../development/README)
- [Deployment Guide](../deployment/README)
- [Security Guide](../security/README)
- [Operations Guide](../operations/README)

==> llm-docs-0.1.5.tgz: package/docs/testing/strategies.md <==
---
id: testing-strategies
---
# Testing Strategies Guide

<!-- title: Testing Strategies Guide -->
<!-- description: Practical guide for testing strategies in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers practical testing strategies for the LLM Platform Platform, including unit testing, integration testing, end-to-end testing, and performance testing.

## 🎯 Drupal TDD Standardization

### Objective

Standardize Test-Driven Development (TDD) across all Drupal modules using ReelCode integration for consistent code quality and testing patterns.

### Current State Analysis

#### Modules Requiring TDD Standardization

1. **✅ ai_swarm_intelligence** - COMPLETED (100% TDD compliant)
2. **🔄 drupal_migration_plus** - IN PROGRESS (dual test structure)
3. **📋 field_eca_bridge** - NEEDS TDD structure
4. **📋 llm** - NEEDS TDD structure  
5. **📋 langchain** - NEEDS TDD structure
6. **📋 api_normalization** - NEEDS TDD structure
7. **📋 secure_drupal** - NEEDS TDD structure
8. **📋 alternative_services** - NEEDS TDD structure
9. **📋 blockchain_manager** - NEEDS TDD structure
10. **📋 experience_builder_converter** - NEEDS TDD structure
11. **📋 mcp_client_extras** - NEEDS TDD structure
12. **📋 marketplace_integration** - NEEDS TDD structure
13. **📋 native_ai** - NEEDS TDD structure
14. **📋 recipe_onboarding** - NEEDS TDD structure

### ReelCode Integration Strategy

#### Phase 1: ReelCode Enhancement for Drupal TDD

```typescript
// Extend ReelCode with Drupal-specific validators
export interface DrupalTDDConfig extends StandardsConfig {
  drupal: {
    moduleStructure: boolean;
    testCoverage: number; // 95% minimum
    tddWorkflow: boolean;
    serviceInjection: boolean;
    ecaIntegration: boolean;
  };
  php: {
    psr12: boolean;
    drupalStandards: boolean;
    phpstan: boolean;
  };
}
```

#### Phase 2: Automated TDD Structure Generation

```bash
# Generate standardized TDD structure for any module
reelcode drupal:generate-tdd-structure --module=module_name

# Validate TDD compliance
reelcode drupal:validate-tdd --module=module_name

# Generate missing tests
reelcode drupal:generate-tests --module=module_name --coverage=95
```

### Standardized TDD Structure

#### Every Module Must Have

```
module_name/
├── module_name.info.yml
├── module_name.module
├── module_name.services.yml
├── phpunit.xml.dist
├── composer.json
├── README.md
├── src/
│   ├── Service/
│   │   ├── Controller/
│   │   ├── Form/
│   │   ├── Plugin/
│   │   │   └── ECA/
│   │   │       ├── Action/
│   │   │       └── Condition/
│   │   └── Entity/
│   ├── tests/
│   │   ├── src/
│   │   │   ├── Unit/
│   │   │   │   ├── Service/
│   │   │   │   ├── Plugin/
│   │   │   │   └── Entity/
│   │   │   ├── Kernel/
│   │   │   ├── Functional/
│   │   │   └── Integration/
│   │   ├── utils/
│   │   └── bootstrap.php
│   ├── config/
│   ├── css/
│   ├── js/
│   └── templates/
```

#### Test File Naming Conventions

- **Unit Tests**: `{ClassName}Test.php`
- **Integration Tests**: `{Feature}IntegrationTest.php`
- **Functional Tests**: `{Feature}FunctionalTest.php`
- **TDD Utilities**: `{Purpose}TddRunner.php`

### Implementation Plan

#### Step 1: Complete drupal_migration_plus TDD Standardization

- [ ] Consolidate dual test structure (`__tests__/` + `tests/`)
- [ ] Organize tests by type (Unit, Kernel, Functional, Integration)
- [ ] Standardize PHPUnit configuration
- [ ] Create TDD utilities

#### Step 2: Enhance ReelCode for Drupal TDD

- [ ] Add Drupal-specific validation rules
- [ ] Create TDD structure generator
- [ ] Add test coverage validation
- [ ] Integrate with PHPStan and PHPCS

#### Step 3: Automated Module Standardization

- [ ] Create script to standardize all remaining modules
- [ ] Generate missing test structures
- [ ] Validate TDD compliance
- [ ] Create CI/CD integration

#### Step 4: Cross-Module Integration Testing

- [ ] Create integration tests between modules
- [ ] Test ECA workflow integration
- [ ] Validate AI platform integration
- [ ] Performance testing

### Quality Gates

#### TDD Compliance Requirements

- ✅ **95%+ Test Coverage** for all modules
- ✅ **Test-First Development** for all new features
- ✅ **Proper Test Organization** by type
- ✅ **Service Dependency Injection** validation
- ✅ **ECA Integration** testing
- ✅ **PHPStan Level 8** compliance
- ✅ **PSR-12 + Drupal Standards** compliance

#### ReelCode Integration

- ✅ **Automated TDD Structure** generation
- ✅ **Test Coverage** validation
- ✅ **Code Quality** enforcement
- ✅ **CI/CD Integration** validation

### Next Actions

1. **Complete drupal_migration_plus standardization** (current)
2. **Enhance ReelCode with Drupal TDD capabilities**
3. **Create automated standardization script**
4. **Apply to all remaining modules**
5. **Implement cross-module integration testing**

### Success Metrics

- **100% TDD Compliance** across all modules
- **95%+ Test Coverage** maintained
- **Consistent Development Patterns** across team
- **Automated Quality Enforcement** via ReelCode
- **Faster Development Velocity** with TDD workflow

### ReelCode CLI Commands for TDD

```bash
# Validate TDD compliance
reelcode drupal validate-tdd -m /path/to/module -c 95

# Generate TDD structure
reelcode drupal generate-tdd-structure -m /path/to/module

# Standardize all modules in directory
reelcode drupal standardize-all -d /path/to/modules

# Generate missing tests
reelcode drupal generate-tests -m /path/to/module -t unit
```

### Integration with CI/CD

```bash
# In your CI pipeline
npm install -g @bluefly/reelcode
reelcode drupal validate-tdd -m ./modules/custom/my_module -c 95
```

---

## Unit Testing

### Agent Testing

```typescript
// src/services/agent.service.spec.ts
describe('AgentService', () => {
  let service: AgentService;
  let modelService: ModelService;
  let memoryService: MemoryService;
  
  beforeEach(async () => {
    const module = await Test.createTestingModule({
      providers: [
        AgentService,
        {
          provide: ModelService,
          useValue: {
            execute: jest.fn(),
            train: jest.fn()
          }
        },
        {
          provide: MemoryService,
          useValue: {
            store: jest.fn(),
            search: jest.fn()
          }
        }
      ]
    }).compile();
    
    service = module.get(AgentService);
    modelService = module.get(ModelService);
    memoryService = module.get(MemoryService);
  });
  
  describe('execute', () => {
    it('should execute agent successfully', async () => {
      const agentId = 'test-agent';
      const input = 'Hello, agent!';
      const config = { model: 'claude-3-opus' };
      
      const expectedResult = {
        output: 'Hello, human!',
        tokens: 10
      };
      
      jest.spyOn(modelService, 'execute')
        .mockResolvedValue(expectedResult);
      
      const result = await service.execute(agentId, input, config);
      
      expect(result).toEqual(expectedResult);
      expect(modelService.execute)
        .toHaveBeenCalledWith(agentId, input, config);
      expect(memoryService.store)
        .toHaveBeenCalledWith(agentId, expectedResult);
    });
    
    it('should handle execution errors', async () => {
      const agentId = 'test-agent';
      const input = 'Hello, agent!';
      const config = { model: 'claude-3-opus' };
      
      const error = new Error('Model error');
      
      jest.spyOn(modelService, 'execute')
        .mockRejectedValue(error);
      
      await expect(service.execute(agentId, input, config))
        .rejects.toThrow('Model error');
      
      expect(memoryService.store)
        .not.toHaveBeenCalled();
    });
  });
});
```

### Model Testing

```typescript
// src/services/model.service.spec.ts
describe('ModelService', () => {
  let service: ModelService;
  let provider: ModelProvider;
  
  beforeEach(async () => {
    const module = await Test.createTestingModule({
      providers: [
        ModelService,
        {
          provide: 'MODEL_PROVIDER',
          useValue: {
            execute: jest.fn(),
            train: jest.fn()
          }
        }
      ]
    }).compile();
    
    service = module.get(ModelService);
    provider = module.get('MODEL_PROVIDER');
  });
  
  describe('execute', () => {
    it('should execute model successfully', async () => {
      const modelId = 'claude-3-opus';
      const input = 'Hello, model!';
      const config = { maxTokens: 100 };
      
      const expectedResult = {
        output: 'Hello, human!',
        tokens: 10
      };
      
      jest.spyOn(provider, 'execute')
        .mockResolvedValue(expectedResult);
      
      const result = await service.execute(modelId, input, config);
      
      expect(result).toEqual(expectedResult);
      expect(provider.execute)
        .toHaveBeenCalledWith(modelId, input, config);
    });
  });
});
```

## Integration Testing

### API Testing

```typescript
// test/integration/agent.test.ts
describe('Agent API', () => {
  let app: INestApplication;
  let agentService: AgentService;
  
  beforeAll(async () => {
    const module = await Test.createTestingModule({
      imports: [AppModule],
      providers: [AgentService]
    }).compile();
    
    app = module.createNestApplication();
    agentService = module.get(AgentService);
    
    await app.init();
  });
  
  afterAll(async () => {
    await app.close();
  });
  
  describe('POST /agents', () => {
    it('should create agent', async () => {
      const input = {
        name: 'Test Agent',
        model: 'claude-3-opus',
        config: {
          maxTokens: 100
        }
      };
      
      const response = await request(app.getHttpServer())
        .post('/agents')
        .send(input)
        .expect(201);
      
      expect(response.body).toMatchObject({
        name: input.name,
        model: input.model,
        status: 'active'
      });
    });
  });
  
  describe('GET /agents/:id', () => {
    it('should get agent', async () => {
      const agent = await agentService.create({
        name: 'Test Agent',
        model: 'claude-3-opus',
        config: {
          maxTokens: 100
        }
      });
      
      const response = await request(app.getHttpServer())
        .get(`/agents/${agent.id}`)
        .expect(200);
      
      expect(response.body).toMatchObject({
        id: agent.id,
        name: agent.name,
        model: agent.model
      });
    });
  });
});
```

### Drupal Testing

```php
// tests/src/Functional/AgentTest.php
namespace Drupal\Tests\bfllm\Functional;

use Drupal\Tests\BrowserTestBase;

class AgentTest extends BrowserTestBase {
  protected $defaultTheme = 'stark';
  
  protected static $modules = ['bfllm'];
  
  public function testAgentCreation() {
    $this->drupalLogin($this->createUser(['administer agents']));
    
    $this->drupalGet('/admin/config/bfllm/agents/add');
    
    $this->submitForm([
      'name' => 'Test Agent',
      'model' => 'claude-3-opus',
      'config[max_tokens]' => 100
    ], 'Save');
    
    $this->assertSession()
      ->pageTextContains('Agent Test Agent has been created.');
  }
  
  public function testAgentExecution() {
    $agent = $this->createAgent([
      'name' => 'Test Agent',
      'model' => 'claude-3-opus',
      'config' => [
        'max_tokens' => 100
      ]
    ]);
    
    $this->drupalGet("/agent/{$agent->id()}/execute");
    
    $this->submitForm([
      'input' => 'Hello, agent!'
    ], 'Execute');
    
    $this->assertSession()
      ->pageTextContains('Execution successful');
  }
}
```

## End-to-End Testing

### Playwright Tests

```typescript
// tests/e2e/agent.spec.ts
import { test, expect } from '@playwright/test';

test.describe('Agent UI', () => {
  test.beforeEach(async ({ page }) => {
    await page.goto('/agents');
  });
  
  test('should create agent', async ({ page }) => {
    await page.click('button:has-text("Create Agent")');
    
    await page.fill('input[name="name"]', 'Test Agent');
    await page.selectOption('select[name="model"]', 'claude-3-opus');
    await page.fill('input[name="maxTokens"]', '100');
    
    await page.click('button:has-text("Save")');
    
    await expect(page.locator('text=Agent created successfully'))
      .toBeVisible();
  });
  
  test('should execute agent', async ({ page }) => {
    await page.click('text=Test Agent');
    
    await page.fill('textarea[name="input"]', 'Hello, agent!');
    await page.click('button:has-text("Execute")');
    
    await expect(page.locator('text=Execution successful'))
      .toBeVisible();
    
    await expect(page.locator('text=Hello, human!'))
      .toBeVisible();
  });
});
```

### Cypress Tests

```typescript
// cypress/integration/agent.spec.ts
describe('Agent UI', () => {
  beforeEach(() => {
    cy.visit('/agents');
  });
  
  it('should create agent', () => {
    cy.get('button').contains('Create Agent').click();
    
    cy.get('input[name="name"]').type('Test Agent');
    cy.get('select[name="model"]').select('claude-3-opus');
    cy.get('input[name="maxTokens"]').type('100');
    
    cy.get('button').contains('Save').click();
    
    cy.contains('Agent created successfully')
      .should('be.visible');
  });
  
  it('should execute agent', () => {
    cy.contains('Test Agent').click();
    
    cy.get('textarea[name="input"]')
      .type('Hello, agent!');
    cy.get('button').contains('Execute').click();
    
    cy.contains('Execution successful')
      .should('be.visible');
    
    cy.contains('Hello, human!')
      .should('be.visible');
  });
});
```

## Performance Testing

### Load Testing

```typescript
// tests/performance/agent.load.ts
import { check } from 'k6';
import http from 'k6/http';

export const options = {
  stages: [
    { duration: '1m', target: 50 },
    { duration: '3m', target: 50 },
    { duration: '1m', target: 0 }
  ],
  thresholds: {
    http_req_duration: ['p(95)<500'],
    http_req_failed: ['rate<0.01']
  }
};

export default function() {
  const response = http.post(
    'http://localhost:3000/agents/execute',
    JSON.stringify({
      agent_id: 'test-agent',
      input: 'Hello, agent!',
      config: {
        model: 'claude-3-opus',
        max_tokens: 100
      }
    }),
    {
      headers: {
        'Content-Type': 'application/json'
      }
    }
  );
  
  check(response, {
    'status is 200': (r) => r.status === 200,
    'response time < 500ms': (r) => r.timings.duration < 500
  });
}
```

### Stress Testing

```typescript
// tests/performance/agent.stress.ts
import { check } from 'k6';
import http from 'k6/http';

export const options = {
  stages: [
    { duration: '2m', target: 100 },
    { duration: '5m', target: 100 },
    { duration: '2m', target: 200 },
    { duration: '5m', target: 200 },
    { duration: '2m', target: 300 },
    { duration: '5m', target: 300 },
    { duration: '2m', target: 0 }
  ],
  thresholds: {
    http_req_duration: ['p(95)<1000'],
    http_req_failed: ['rate<0.05']
  }
};

export default function() {
  const response = http.post(
    'http://localhost:3000/agents/execute',
    JSON.stringify({
      agent_id: 'test-agent',
      input: 'Hello, agent!',
      config: {
        model: 'claude-3-opus',
        max_tokens: 100
      }
    }),
    {
      headers: {
        'Content-Type': 'application/json'
      }
    }
  );
  
  check(response, {
    'status is 200': (r) => r.status === 200,
    'response time < 1s': (r) => r.timings.duration < 1000
  });
}
```

## Best Practices

1. **Unit Testing**
   - Test business logic
   - Mock dependencies
   - Use meaningful assertions
   - Maintain coverage

2. **Integration Testing**
   - Test API endpoints
   - Test database
   - Test external services
   - Use test database

3. **End-to-End Testing**
   - Test user flows
   - Test UI components
   - Test error handling
   - Use test data

4. **Performance Testing**
   - Set realistic goals
   - Monitor resources
   - Analyze results
   - Update thresholds

## Related Documentation

- [Architecture Guide](../architecture/README)
- [Development Guide](../development/README)
- [Deployment Guide](../deployment/README)
- [Security Guide](../security/README)
- [Operations Guide](../operations/README)
